@@ -7,20 +7,15 @@ if(DEBUG_FLAG) {
   warning("in pbmcmapply.R: disable these lines before publishing package!")
 }
 
+#' @importFrom parallel mcmapply
+#' @export
 pbmcmapply <- function(FUN, ..., MoreArgs = NULL, mc.style = "ETA", mc.substyle = NA,
-                       mc.cores =getOption("mc.cores", 2L),
+                       mc.cores = getOption("mc.cores", 2L),
                        ignore.interactive = getOption("ignore.interactive", F),
-                       max.vector.size = getOption("max.vector.size", 1024L),
                        mc.preschedule = TRUE, mc.set.seed = TRUE,
                        mc.cleanup = TRUE) {
 
-  # Set up maximun global size for the future package
-  .setMaxGlobalSize(max.vector.size)
-
-  # Set up plan
-  originalPlan <- plan("list")
-  on.exit(plan(originalPlan))
-  plan(multiprocess)
+  FUN <- match.fun(FUN)
 
   # Get the max length of elements in ...
   length <- max(mapply(function(element) {
@@ -30,7 +25,9 @@ pbmcmapply <- function(FUN, ..., MoreArgs = NULL, mc.style = "ETA", mc.substyle
       return(nrow(element))
     }
   }, list(...)))
-  .verifyLength(length, "max element has a length of zero.")
+  if (!.verifyLength(length)) {
+    return(list())
+  }
 
   # If not in interactive mode, just pass to mclapply
   if (!interactive() & !ignore.interactive) {
@@ -71,7 +68,7 @@ pbmcmapply <- function(FUN, ..., MoreArgs = NULL, mc.style = "ETA", mc.substyle
   progressFifo <- .establishFifo(tempfile())
   on.exit(close(progressFifo), add = T)
 
-  progressMonitor <- futureCall(function(FUN, ..., MoreArgs, mc.cores) {
+  progressMonitor <- .customized_mcparallel({
     # Get results
     result <- mcmapply(function(...) {
       res <- FUN(...)
@@ -90,18 +87,21 @@ pbmcmapply <- function(FUN, ..., MoreArgs = NULL, mc.style = "ETA", mc.substyle
     # Close the FIFO connection
     close(progressFifo)
 
-    return(result)
-  }, globals = list(progressFifo = progressFifo), args = list(FUN, ..., MoreArgs = MoreArgs, mc.cores = mc.cores))
+    result
+  })
+
+  # clean up processes on exit
+  on.exit(.cleanup(progressMonitor$pid), add = T)
 
   hasErrorInProgress <- .updateProgress(length, progressFifo, mc.style, mc.substyle)
 
-  # Retrieve the result from the future
-  results <- value(progressMonitor)
+  # Retrieve the result
+  results <- mccollect(progressMonitor$pid)[[as.character(progressMonitor$pid)]]
 
-  # Check if errors happened in the future
+  # Check if errors happened
   if (hasErrorInProgress) {
     warning("scheduled cores encountered errors in user code")
   }
 
-  return(results)
+  return(.suppressSelectChildrenWarning(results))
 }
